#!/usr/bin/perl -w
#This script downloads a file from a remote host using wget
#Author: William Hsiao
#Date: March 10 2005
#Modifed May 31, 2006
#Add backup functionality: the last 3 months worth of update will be kept
#Older updates will be deleted to save space
#WARNING: never run this script as root since it has a few automatic file
#deletion operation!

use strict;
use Time::localtime;
use Time::Local;
use File::stat;
use Getopt::Long;
use LWP::Simple;

my $cur_time = localtime;
my ($DAY, $MONTH, $YEAR) = ($cur_time->mday, $cur_time->mon+1, $cur_time->year+1900);
if ($DAY<10){$DAY='0'.$DAY;}
if ($MONTH<10){$MONTH='0'.$MONTH;}

my $host = 'ftp://ftp.ncbi.nih.gov';
my $remotedir = 'genomes/Bacteria/';
my $parentdir  = '/home/shared/NCBI_genomes/curated/';
my $prefix = 'Bacteria';
my $parameters ='';
my $clean = 0; #default is not to clean older directories
my $overwrite =0; #default is not to overwrite
my $get_gprj = 0; #default is not to get the organism info and complete genome files
my $logfile = $parentdir."/log/NCBI_FTP.log";

GetOptions ("host=s"      => \$host,
	    "remotedir=s" => \$remotedir,
	    "localdir=s"  => \$parentdir,
	    "param=s"     => \$parameters,
            "logfile=s"   => \$logfile,
	    "overwrite"   => \$overwrite,
	    "clean"       => \$clean,
	    "get_gprj"    => \$get_gprj
	    );

die "The local parent directory doesn't exist, please create it first\n" unless (-e $parentdir);
my $localdir = $parentdir."$prefix\_$YEAR\-$MONTH\-$DAY";

#s parameters: turn on mirroring; no host directory;
# non-verbose; exclude .val files; .listing file kept;
if ($parameters eq ''){
    $parameters ="--passive-ftp -P $localdir -m -nH --cut-dirs=2 -w 2 -nv -K -R val -a $logfile ";
}
#Create the log file if it doesn't exist already
&createfile($logfile) unless (-e $logfile);

#if the output dir exists, put a warning in log and overwrite if asked
if (-e $localdir){  
    if ($overwrite){
	&writetolog("Directory $localdir already exists, files is being overwritten\n");
	&runwget;
	if ($get_gprj) { &get_genomeprojfiles;}
    }
    else{
	&writetolog("Update failed on $YEAR-$MONTH-$DAY because directory already exists \n");
    }
}
else {
    &runwget;
    if ($get_gprj) { &get_genomeprojfiles;}
}

#create a sym link from Bacteria to the new directory
unlink "$parentdir$prefix" if (-l "$parentdir$prefix");
symlink "$localdir", "$parentdir$prefix" or &writetolog("Unable to create symlink from $localdir to $parentdir$prefix\n");

if ($clean) {
    &cleandirectory($cur_time, $parentdir);
}
    
#the simple subroutine that makes a system call to wget
# see http://www.gnu.org/software/wget/manual/wget.html for wget manual
sub runwget{
    my $status =1;
    my $count =0;
    while ($status != 0 && $count<5){
	$status = system ("wget $parameters $host/$remotedir/");
	sleep 120 unless $status == 0;
	$count++;
    }
}

sub get_genomeprojfiles{
    print "Hello";
    my $content = get('http://www.ncbi.nih.gov/genomes/lproks.cgi?view=0&dump=selected');
    my $content2 = get('http://www.ncbi.nih.gov/genomes/lproks.cgi?view=1&dump=selected');
    open (ORGINFO, ">$localdir/NCBI_orginfo.txt") or die "can't create file $localdir/NCBI_orginfo.txt";
    open (COMPGEN, ">$localdir/NCBI_completegenomes.txt") or die "can't create file $localdir/NCBI_completegenomes.txt";
    print ORGINFO $content;
    print COMPGEN $content2;
    close ORGINFO;
    close COMPGEN;
}

#delete backup directories that are older than 90 days
sub cleandirectory {
    my $curdate = shift;
    my $dir = shift;
    opendir(DIR, $dir);
    my $file;
    while (defined($file = readdir(DIR))){
	next if $file =~/^\.\.?$/;
	if (-d $file){
	    my $filestat = stat($file);
	    my $filechangeinode = $filestat->ctime;
	    my $filechangedate = localtime($filechangeinode);
	    #90 days has 7776000 seconds
	    if ($curdate - $filechangedate > 7776000){
		system ("rm -rf $file");
	    }
	}
    }
}

sub writetolog {
    my $message = shift;
    open (LOG, ">>$logfile") or die "can not open log file $logfile\n";
    print LOG $message;
    close LOG;
}

sub createfile {
    my $filename = shift;
    my $createstatus = system ("touch $filename");
    if ($createstatus==0){ chmod 0666, $filename; }
    else{writetolog ("File $filename cannot be created\n");}    
}
