package MicrobeDB;

#MicrobeDB is the parent class for all classes in the microDB project
#perldoc MicrobeDB - for more information (or see end of this file)

use strict;
use warnings;
use Carp;

our $AUTOLOAD;

my @GENERAL_FIELDS = qw(comment);


sub new {
	my ( $class, %arg ) = @_;

	#Bless an anonymous empty hash
	my $self = bless {}, $class;

	#Fill all of the keys with the fields
	foreach (@GENERAL_FIELDS) {
		$self->{$_} = undef;
	}

	#Set each attribute that is given as an arguement
	foreach ( keys(%arg) ) {
		$self->$_( $arg{$_} );
	}
	
	return $self;
}


# This takes the place of methods to set or get the value of an attribute
sub AUTOLOAD {
    my ($self,$newvalue) = @_;
    #get the unknown method call
    my $attr = $AUTOLOAD;

    #Keep only the method name
    $attr =~ s/.*:://;    

    #Die if the key does not already exist in the hash
    unless (exists($self->{$attr})){
	croak "No such attribute '$attr' exists in the class ";
    }

    # Turn off strict references to enable "magic" AUTOLOAD speedup
    no strict 'refs';

    # define subroutine
    *{$AUTOLOAD} = sub {my($self,$newvalue)=@_;
			$self->{$attr}=$newvalue if defined($newvalue);
			return $self->{$attr}};    

    # Turn strict references back on
    use strict 'refs';

    #Set the new value for the attribute if available
    $self->{$attr} = $newvalue if defined($newvalue);

    #Always return the current value for the attribute
    return $self->{$attr};
}



#returns an array of all field names for this class
sub all_fields{
  my ($self) = @_;
  return keys(%$self);
}

#set all the fields in the object when given a hash
sub set_hash{
	my($self,%hash)=@_;
	foreach(keys(%hash)){
		$self->$_($hash{$_});
	}
}

#returns a hash of the complete object
sub get_hash{
	my ($self) = @_;
	return %{$self};
}

#print object to tab-delimited file
#TODO: need to update the print function to use filenames
sub print_obj{
	my ($self, $filename) =@_;
	my @print_line = join('\t',values(%$self));
	print @print_line;
}

#Anything put in this method will be run when the object is destroyed
sub DESTROY{
}


1;

__END__

=head1 NAME

Replicon: contains features that are associated with a single replicon (chromosome or plasmid) within an organism

=head1 Synopsis

=head1 AUTHOR

Morgan Langille

=head1 Date Created

June 2nd, 2006

=cut


